export const share = (e) => {
  const shareUrl = location.href;
  const title = document.title;

  switch (e.currentTarget.getAttribute('share')) {
    case 'facebook': {
      openFacebook({ title, shareUrl });
      break;
    }
    case 'twitter': {
      openTwitter({ title, shareUrl });
      break;
    }
    case 'whatsapp': {
      openWhatsapp({ title, shareUrl });
      break;
    }
    default:
      throw new Error('Unknown share target');
  }
};

const features = ({ height }: { height: number }): string => {
  return [
    ['menubar', 'no'],
    ['toolbar', 'no'],
    ['resizable', 'yes'],
    ['scrollbars', 'yes'],
    ['width', 600],
    ['height', height],
  ]
    .map((feature) => feature.join('='))
    .join(',');
};

const openTwitter = ({ title, shareUrl }: { title: string; shareUrl: string }) => {
  const text = `Check out ${title} at #{$ hashtag $}: ${shareUrl}`;
  const url = `https://twitter.com/intent/tweet?text=${encodeURIComponent(text)}`;
  window.open(url, 'share', features({ height: 275 }));
};

const openFacebook = ({ title, shareUrl }: { title: string; shareUrl: string }) => {
  const url = `https://www.facebook.com/sharer.php?u=${encodeURIComponent(
    shareUrl
  )}&t=${encodeURIComponent(title)}`;
  window.open(url, 'share', features({ height: 775 }));
};

const openWhatsapp = ({ title, shareUrl }: { title: string; shareUrl: string }) => {
  const url = `whatsapp://send?text=${encodeURIComponent(title)}%0A%0A${encodeURIComponent(
    shareUrl
  )}`;
  window.open(url, 'share', features({ height: 775 }));
};
